"""Ferda verificaiton utils."""
from typing import Any, cast

from asgiref.sync import sync_to_async
from django.core.cache import cache
from django.http import HttpRequest as Request
from ferda.backend import CONTACT, LOGGER
from ferda.constants import LOGGER_SESSION_KEY
from fred_types import ContactId
from grill.logger import LogEntry
from regal import Contact

from .constants import LogEntryType


async def create_log_entry(
    request: Request, log_entry_type: LogEntryType, /, **kwargs: Any
) -> LogEntry:
    """Create log entry."""
    log_entry = LOGGER.create(
        log_entry_type,
        source_ip=request.META.get("REMOTE_ADDR", ""),
        session_id=await sync_to_async(request.session.get)(LOGGER_SESSION_KEY),
        **kwargs,
    )
    return log_entry


async def get_contact_info(contact_id: ContactId) -> Contact:
    """Get contact info from cache or from backend."""
    cache_key = f"ferda_verification:contact:{contact_id}"
    if cache_value := cache.get(cache_key):
        return cast(Contact, cache_value)

    contact = await CONTACT.get_contact_info(contact_id)
    cache.set(cache_key, contact)
    return contact
